//Labor day weekend, gamedev.net contest "Sticky" theme entry
//By Adam Sepanski http://adamsepanski.com 9/1/07

/*Notes:
 * on level up spawn time decreases, martian speed increases, perhaps goo life decreases?
 */

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace StickyGame
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    ///

    public class Game1 : Microsoft.Xna.Framework.Game
    {
        //constants
        const int MAX_GOO = 200;
        const int MAX_INVADERS = 250;
        const int MAX_SPLATS = 15;//number of invader splats
        const int SPAWN_INTERVAL=3300;//in milliseconds
        const float INVADER_GOOP_SLOWDOWN = .0008f;
        const int LEVEL_TIMER = 20000;//in milliseconds
        const int LEVEL_SPAWN_MODIFIER = 100;//in milliseconds, how much faster they spawn each level
        const float PROJECTILE_SPEED = .3f;//used to slow down projectiles

        #region Game State Variables
        //game state variables
        bool mLevelReset = true;
        int mLevelCounter = 0;
        int mLevel = 1;
        Random rand = new Random();
        bool isGameOver = false;
        bool isGameStarting = true;
        bool isMusicOn = true;
        bool isSoundOn = true;
        bool hasMusicStarted = false;
        #endregion

        #region member functions, globalesque variables
        GraphicsDeviceManager graphics;
        ContentManager content;

        TextureCreationParameters tcp;
        SpriteBatch mSpriteBatch;
        KeyboardState mKeyboardState, mOldKeyboardState;
        MouseState mMouseState, mOldMouseState;
        SpriteFont mFont, mFontHeader,mFontTitle;
        string mText="";
        string mGameOverText = "";
        string mGameOverStats = "";
        string mGameTitle = "";

        Texture2D mBackground;
        Texture2D mBackgroundMenu;
        Texture2D mInstructions;
        Cursor mCursor;

        Audio mAudio;
        Cue mMusicCue;

        Turret mTurret;
        Projectile mProjectile;
        StickyGoo[] mGoo = new StickyGoo[MAX_GOO];
        int mGooCounter=0;
        Invader[] mInvader = new Invader[MAX_INVADERS];
        int mInvaderCounter = 0;
        long mSpawnInterval = 0;
        Splat[] mSplat = new Splat[MAX_SPLATS];
        int mSplatCounter = 0;

        int mEmbassyHealth = 100;
        int mScore = 0;
        int mNumOfInvaders = 0;
        int mNumOfGoos = 0;
        #endregion

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
            tcp = TextureCreationParameters.Default;
            Window.Title = "Goo Defender";

        }

        #region Initializations
        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferredBackBufferHeight = 600;
            graphics.IsFullScreen = false;

            tcp.ColorKey = Color.Fuchsia;

            this.IsMouseVisible = false;

            mText = "";

            mSpriteBatch=new SpriteBatch(graphics.GraphicsDevice);

            mKeyboardState = new KeyboardState();
            mOldKeyboardState = new KeyboardState();
            mMouseState = new MouseState();
            mOldMouseState = new MouseState();

            mBackground=content.Load<Texture2D>("./Images/StickyGameBG")as Texture2D;
            mInstructions = content.Load<Texture2D>("./Images/StickyInstructions") as Texture2D;
            mBackgroundMenu = content.Load<Texture2D>("./Images/StickyGameMenu") as Texture2D;

            mAudio = new Audio();
            mMusicCue = mAudio.mSoundBank.GetCue("StickyGameMusic") ;

            mCursor = new Cursor(graphics.GraphicsDevice, content);

            mTurret = new Turret(graphics.GraphicsDevice, content);
            mProjectile= new Projectile(graphics.GraphicsDevice,content);

            for (int index = 0; index < MAX_GOO; index++)
            {
                mGoo[index] = new StickyGoo(graphics.GraphicsDevice, content);
            }

            for (int index = 0; index < MAX_INVADERS; index++)
            {
                mInvader[index] = new Invader(graphics.GraphicsDevice, content);
            }

            for (int index = 0; index < MAX_SPLATS; index++)
            {
                mSplat[index] = new Splat(graphics.GraphicsDevice, content);
            }

            base.Initialize();
        }
        #endregion

        #region Content load and unload, very rarely used by myself
        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content
            }

            // TODO: Load any ResourceManagementMode.Manual content
            mFont=content.Load<SpriteFont>("./Fonts/StickyFont") as SpriteFont;
            mFontHeader = content.Load<SpriteFont>("./Fonts/HeaderFont") as SpriteFont;
            mFontTitle = content.Load<SpriteFont>("./Fonts/TitleFont") as SpriteFont;
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent)
            {
                // TODO: Unload any ResourceManagementMode.Automatic content
                content.Unload();
            }

            // TODO: Unload any ResourceManagementMode.Manual content
        }
        #endregion

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // TODO: Add your update logic here
            #region keyboard states and some option keys
            mOldKeyboardState = mKeyboardState;
            mKeyboardState = Keyboard.GetState();
            mOldMouseState = mMouseState;
            mMouseState = Mouse.GetState();

            // Allows the game to exit
            if (mKeyboardState.IsKeyDown(Keys.Escape))
            { this.Exit(); }
            //toggle fullscreen for debugging
            if (KeyPressed(mKeyboardState,mOldKeyboardState,Keys.F))
            { graphics.ToggleFullScreen();}
            if (KeyPressed(mKeyboardState, mOldKeyboardState, Keys.M))
            {
                if (isMusicOn)
                {
                    isMusicOn = false;
                    mMusicCue.Stop(AudioStopOptions.AsAuthored);
                    hasMusicStarted = true;
                }
                else if(!isMusicOn)
                {
                    isMusicOn = true;
                    hasMusicStarted = false;
                }
            }
            if (KeyPressed(mKeyboardState, mOldKeyboardState, Keys.S))
            { isSoundOn = !isSoundOn; }
            #endregion

            #region Audio and Music
            mAudio.Update();

            if (isMusicOn && !hasMusicStarted)
            {

                if (mMusicCue.IsStopped || mMusicCue.IsStopping)
                {
                }
                else
                    mMusicCue.Play();
                hasMusicStarted = true;
            }
            #endregion
            mCursor.Update(mMouseState);

            #region Game State Blocks...These are the starting and losing blocks
            if (isGameStarting)
            {
                mGameTitle = "Goo Defender";
                
                if (mKeyboardState.IsKeyDown(Keys.Space))
                {
                    isGameStarting = false;
                }
            }
            else if (isGameOver)
            {
                mGameOverText = "The Martians invaded Earth's colony, \nsetting the planet back by decades....";
                mGameOverStats="        You made it to Level " + mLevel + "\n       Your final score was " + mScore + "\nYou shot at " +
                    mNumOfInvaders + " invaders with " + mNumOfGoos + " goo shots."+
                    "\n\n\n\n      Press Spacebar to Continue.....";

                if (mKeyboardState.IsKeyDown(Keys.Space))
                {
                    isGameOver = false;
                    ResetObjects();
                    mLevelReset = true;
                }
            }
            #endregion
            //regular game loop
            #region Normal Game running loop
            else
            {
                if (mLevelReset)
                {
                    mSpawnInterval = gameTime.TotalGameTime.Minutes * 60 * 1000 + gameTime.TotalGameTime.Seconds * 1000 + gameTime.TotalGameTime.Milliseconds;
                    mLevelCounter = gameTime.TotalGameTime.Minutes * 60 * 1000 + gameTime.TotalGameTime.Seconds * 1000 + gameTime.TotalGameTime.Milliseconds;
                    mLevelReset = false;

                    //ResetObjects();
                }

                if (gameTime.TotalGameTime.Minutes * 60 * 1000 + gameTime.TotalGameTime.Seconds * 1000 + gameTime.TotalGameTime.Milliseconds - mLevelCounter >=
                    LEVEL_TIMER)
                {
                    mLevel++;
                    mLevelReset = true;
                }

                mTurret.Update(new Vector2(mCursor.Properties.X + mCursor.Properties.Width / 2, mCursor.Properties.Y + mCursor.Properties.Height / 2 - 5));

                #region goo turret firing logic
                //if the goo turret is fired
                if (LeftMouseClicked(mMouseState, mOldMouseState))
                {
                    if (mProjectile.FiringMode)
                    {
                        mProjectile.Fire(
                            mTurret.Properties.X, mTurret.Properties.Y, mTurret.RotationAngle, PROJECTILE_SPEED);
                        mProjectile.FiringMode = false;
                        mCursor.ChangeFiringMode(mProjectile.FiringMode);
                        mNumOfGoos++;
                        if (isSoundOn)
                        { mAudio.Play("GooFire"); }
                    }
                    else
                    {
                        //if we're on splat mode, fire the goo when mouse clicked
                        mGoo[mGooCounter].Fire((int)mProjectile.Properties.fX, (int)mProjectile.Properties.fY, mLevel);

                        mGooCounter++;//iterate goo counter in array
                        if (mGooCounter == MAX_GOO)
                        {
                            mGooCounter = 0;
                        }

                        mProjectile.Properties.Visible = false;
                        mProjectile.FiringMode = true;
                        mCursor.ChangeFiringMode(mProjectile.FiringMode);
                        if (isSoundOn)
                        { mAudio.Play("GooSplat"); }
                    }
                }
                #endregion

                mProjectile.Update();

                #region invader spawning logic
                long interval = gameTime.TotalGameTime.Milliseconds + gameTime.TotalGameTime.Seconds * 1000 + gameTime.TotalGameTime.Minutes * 60 * 1000 - mSpawnInterval;
                //mText = "Level "+mLevel+": " + interval+" gametime " + (gameTime.TotalGameTime.Milliseconds+gameTime.TotalGameTime.Seconds*1000+gameTime.TotalGameTime.Minutes*60*1000);
                if (interval >= SPAWN_INTERVAL - (LEVEL_SPAWN_MODIFIER * mLevel))
                {
                    mInvader[mInvaderCounter].Spawn(
                        Window.ClientBounds.Width,
                        rand.Next(0, Window.ClientBounds.Height - (int)mInvader[mInvaderCounter].Properties.fHeight));
                    mNumOfInvaders++;
                    mInvaderCounter++;
                    if (mInvaderCounter >= MAX_INVADERS)
                    { mInvaderCounter = 0; }
                    mSpawnInterval = gameTime.TotalGameTime.Milliseconds + gameTime.TotalGameTime.Seconds * 1000 + gameTime.TotalGameTime.Minutes * 60 * 1000;
                }
                #endregion

                #region invader collision logic, with goo and embassy
                foreach (Invader index in mInvader)
                {
                    //update invader positions
                    index.Update();
                    //check for collisions with goo splat and slow down if necessary (only check if both objects are visible on screen)
                    if (index.Properties.Visible)
                    {
                        foreach (StickyGoo gooindex in mGoo)
                        {
                            if (gooindex.Properties.Visible)
                            {
                                if (index.BoundingBox.Intersects(gooindex.BoundingBox))
                                {
                                    index.Properties.fVX += INVADER_GOOP_SLOWDOWN;//each goop pile multiplies the slow down factor
                                }
                            }
                        }
                        /*took this out, this was before when I was doing a slate wipe at each level
                        //slowly gookill the invader
                        if (isGooped)
                        {
                            //index.Life--;
                            //index.Properties.fVX += INVADER_GOOP_SLOWDOWN;//uncomment to have 2+ goops only slow down invader as if one
                        }
                        //change the speed of invader, fast=false if isgooped=true
                        //index.ChangeSpeed(!isGooped);
                         */

                        //wall collisions
                        if (index.Properties.fX <= 130)
                        {
                            mSplat[mSplatCounter].Spawn((int)index.Properties.fX, (int)index.Properties.fY);
                            mSplatCounter++;
                            if (mSplatCounter >= MAX_SPLATS)
                            { mSplatCounter = 0; }

                            mAudio.Play("InvaderSplat");

                            mEmbassyHealth -= index.DamageEmbassy();
                        }
                    }

                }
                #endregion

                foreach (StickyGoo index in mGoo)
                {
                    index.Update();
                }
                foreach (Splat index in mSplat)
                {
                    index.Update();
                }

                if (mEmbassyHealth <= 0)
                {
                    isGameOver = true;
                }
                mScore = (mNumOfInvaders * mLevel * 100) - (mNumOfGoos * 10);
                mText = "Level " + mLevel + "      Embassy Health: " + mEmbassyHealth + "      Score: " + mScore;
            }
            #endregion
            //end normal game loop
            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            mSpriteBatch.Begin(SpriteBlendMode.AlphaBlend);

            mSpriteBatch.Draw(mBackground, new Rectangle(0, 0, Window.ClientBounds.Width, Window.ClientBounds.Height), Color.White);
            mTurret.Draw(mSpriteBatch);
            mProjectile.Draw(mSpriteBatch);
            foreach (StickyGoo index in mGoo)
            {
                index.Draw(mSpriteBatch);
            }

            foreach (Invader index in mInvader)
            {
                index.Draw(mSpriteBatch);
            }

            foreach (Splat index in mSplat)
            {
                index.Draw(mSpriteBatch);
            }

            if (isGameStarting)
            {
                mSpriteBatch.Draw(mBackgroundMenu, new Vector2(0, 0), Color.White);
                mSpriteBatch.DrawString(mFontTitle, mGameTitle, new Vector2(280, 0), Color.DarkBlue);
                mSpriteBatch.Draw(mInstructions, new Vector2(0,0), Color.White);
            }
            if (isGameOver)
            {
                mSpriteBatch.Draw(mBackgroundMenu, new Vector2(0, 0), Color.White);
                mSpriteBatch.DrawString(mFontHeader, mGameOverText, new Vector2(185, 125), Color.DarkBlue);
                mSpriteBatch.DrawString(mFont, mGameOverStats, new Vector2(220, 260), Color.Black);
            }
            mSpriteBatch.DrawString(mFont, mText, new Vector2(200, 0), Color.Black);

            mCursor.Draw(mSpriteBatch);

            mSpriteBatch.End();

            base.Draw(gameTime);
        }

        private bool LeftMouseClicked(MouseState currentmouse, MouseState oldmouse)
        {
            if (currentmouse.LeftButton == ButtonState.Pressed && oldmouse.LeftButton == ButtonState.Released)
                return true;
            return false;
        }

        private bool KeyPressed(KeyboardState currentstate, KeyboardState oldstate, Keys key)
        {
            if (currentstate.IsKeyDown(key) && oldstate.IsKeyUp(key))
                return true;
            return false;
        }

        //called to reset the position and velocity of all objects
        private void ResetObjects()
        {
            //invader
            foreach (Invader index in mInvader)
            {
                index.Properties.Visible = false;
                mInvaderCounter = 0;
            }
            //projectile
            mProjectile.Properties.Visible = false;
            //goo
            foreach (StickyGoo index in mGoo)
            {
                index.Properties.Visible = false;
                mGooCounter = 0;
            }
            //splats
            foreach (Splat index in mSplat)
            {
                index.Properties.Visible = false;
                mSplatCounter = 0;
            }
            mScore = 0;
            mNumOfGoos = 0;
            mNumOfInvaders = 0;
            mEmbassyHealth = 100;
        }
    }
}
