using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Storage;

namespace StickyGame
{
    class Invader
    {
        //invader regular speed and speed when gooped
        const float SPEED = -.40f;
        const float SLOW_SPEED = -.1f;
        const float LEVEL_SPEED_MODIFIER = -.05f;
        const int MAX_LIFE = 50;

        public SpriteProperties Properties = new SpriteProperties();
        public BoundingBox BoundingBox = new BoundingBox();

        private int mLife;
        public int Life
        {
            get
            { return mLife; }
            set
            { mLife = value; }
        }

        //constructor
        public Invader(GraphicsDevice theGraphicsDevice, ContentManager theContentManager)
        {
            Properties.fWidth = 32;
            Properties.fHeight = 48;
            Properties.Visible = false;
            Properties.Texture = theContentManager.Load<Texture2D>("./Images/StickyMartian") as Texture2D;

            mLife = MAX_LIFE;
        }

        public void Update()
        {
            if (Properties.fVX > 0)
            { Properties.fVX = 0; }

            Properties.fX += Properties.fVX;
            Properties.fY += Properties.fVY;

            if (mLife <= 0)
            { Properties.Visible = false; }
            //update the bounding box if visisble
            if (Properties.Visible)
            {
                //we don't want the box to encompass the whole martian, just the legs or so, so we offset by 28
                BoundingBox.Min = new Vector3(Properties.fX, Properties.fY + 32, 0f);
                BoundingBox.Max = new Vector3(Properties.fX + Properties.fWidth, Properties.fY + Properties.fHeight, 0f);
            }
        }

        public void Draw(SpriteBatch theSpriteBatch)
        {
            if (Properties.Visible)
            {
                theSpriteBatch.Draw(Properties.Texture,
                    new Rectangle((int)Properties.fX, (int)Properties.fY, (int)Properties.fWidth, (int)Properties.fHeight),
                    Color.White);
            }
        }

        public void Spawn(int startx, int starty)
        {
            Properties.fX = (float)startx;
            Properties.fY = (float)starty;
            Properties.Visible = true;
            Properties.fVX = SPEED;
            Properties.fVY = 0;
        }

        public void ChangeSpeed(bool fast)
        {
            if (fast)
            { Properties.fVX = SPEED; }
            else
            { Properties.fVX = SLOW_SPEED; }
        }

        public int DamageEmbassy()
        {
            Properties.Visible = false;
            return Math.Abs( (int)(Properties.fVX * 25));
        }
    }
}
